/*
 * BrokerClient.java
 * 
 */

package hotTip;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JPasswordField;
import java.io.*;
import java.util.*;
import java.math.BigInteger;

// socket
import java.net.*;
import javax.net.ssl.*;

//  Crypto
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;
import java.security.interfaces.*;

import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import javax.security.auth.x500.*;

// IAIK security toolkit
import iaik.x509.*;
import iaik.asn1.structures.AlgorithmID;

/**************************************************************
 *
 *
 *
 *
 *
 **************************************************************/
public class BrokerClient {

    /**********************GUI vars *****************************/
    //  The GUI
    BrokerClientPanel _panel;
    BuySellPanel _buySellPanel;
    CardLayout _layout;
    JFrame _appFrame;
    /*****************end GUI vars ******************************/

    /*******************[vars for communication]******************/    
    //SSLSocket caSocket = null;
    //SSLSocket brkrSocket = null;

    Socket caSocket = null;
    Socket brkrSocket = null;

    InputStream is = null;
    OutputStream os = null;
    
    BufferedReader in = null;
    PrintWriter out = null;
    /*************************************************************/

    /*********[client keystore, cert, ... vars]*******************/
    String caHost, brkrHost;        
    int caPort, brkrPort;
    private String userName;
    private char[] pwd;

    boolean firstTime; // true if is first time broker is connecting
                       // to BrokerClient (over duration of Broker's
                       // lifetime == up time); false otherwise
    /**************************************************************/
    private KeyStore clientKeyStore = null;
    private KeyStore caKeyStore = null;
    
    private String MYKEY_ALIAS = null;
    private String cliKSname = null; // keystore name
    private char[] cliKSpwd = null;  // keystore pass

    private iaik.x509.X509Certificate clientCertX509 = null;
    private iaik.x509.X509Certificate signedClientCertX509;
    
    private Principal clientName = null;
    private PublicKey clientPublicKey = null;
    private PrivateKey clientPrivateKey = null;
    private BigInteger serialNumber = null;
    private Principal issuerName = null;
    /**************************************************************/

    /**********************[SSL vars]******************************/
    private KeyManagerFactory kmf;
    private TrustManagerFactory tmf;
    private SSLContext sslContext;    
    private SSLSocketFactory socketFactory;
    /**************************************************************/

    // [you-may-need-later vars]
    //private TrustManager[] myTM;
    //private String SIGNED_CLIENT_KEYSTORE = "signedClientKeys";

    /**************************************************************
     *
     *
     *
     *
     *
     **************************************************************/    
    public BrokerClient() {

       _panel = null;
       _buySellPanel = null;
       _layout = null;
       _appFrame = null;
       
       try {
	   initialize(); } 
       
       catch (Exception e) {
	   System.err.println("BrokerClient error: " + e.getMessage());
	   e.printStackTrace(); }

       _layout.show(_appFrame.getContentPane(), "BrokerClientPanel");
    }
    
    //
    //  initialize
    //  
    //  BrokerClient initialization
    //
     private void initialize() throws Exception {

	 _appFrame = new JFrame("BrokerClient");
	 _layout = new CardLayout();
	 
	 _appFrame.getContentPane().setLayout(_layout);
	 _panel = new BrokerClientPanel(this);
	 _buySellPanel = new BuySellPanel(this);
	 _appFrame.getContentPane().add(_panel, "BrokerClientPanel");
	 _appFrame.getContentPane().add(_buySellPanel,
					"BuySellPanel");   
	 
	 _appFrame.addWindowListener(new WindowAdapter() {
		 public void windowClosing(WindowEvent e) {
		     quit();
		 }
	     });
     }
    
    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/  
    public void run() {
        _appFrame.pack();
        _appFrame.setVisible(true);
    }
    
    
    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/    
    public void startup( String _userName, char[] _pwd,
			 String _cliKSname, char[] _cliKSpwd,
			 String _caHost,int _caPort,
			 String _brkrHost, int _brkrPort,
			 boolean _firstTime ) {
	
	userName = _userName;
	pwd = _pwd;
	cliKSname = _cliKSname;
	cliKSpwd = _cliKSpwd;
	caHost = _caHost;
	caPort = _caPort;
	brkrHost = _brkrHost;
	brkrPort = _brkrPort;	
	firstTime = _firstTime;

	/*************************************************************** 
	 * TO DO:
	 * ------
	 * populate : clientCertX509, clientName, clientPublicKey,...
	 *
	 *  via reading in from client keystore
	 ***************************************************************/

	doCAStuff();
	
	//
	// NB: only continue if you have obtained a signed cert from the CA;
	//
	
	/**************************************************************
	 * Now that you have a certificate signed by the trusted CA...
	 * 
	 *  set up: SSLContext, KeyManagerFactory, TrustManager; 
	 *  init:   SSLContext; 
	 *  get:    a new socketFactory from your SSLContext
	 *
	 **************************************************************/
	
	setUpBrokerConn();
	
	_layout.show(_appFrame.getContentPane(), "BuySellPanel");
	
    }

    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void doCAStuff( ) {		    	
	
	try {
	    caSocket = new Socket( caHost, caPort ); }

	catch( Exception e ) {
	    System.err.println(e.getMessage());
	    e.printStackTrace(); }

	try {
	    
	    // set up input/output streams
	    setUpStreams( caSocket );
	    
	    //
	    // send username, password, cert (X509Certificate) to CA
	    //	    
	    out.println(userName);
	    out.flush();
	    
	    out.println( new String(pwd) );
	    out.flush();
	    
	    // send client cert
	    

	    // send EOF
	    out.println("END");
	    out.flush();
	    
	    String resp = in.readLine();
	    
	    if ( resp == null ) {
		System.err.println("response is null....");
		System.exit(1); }
	    
	    if ( !resp.equals("OK") ) 
		System.exit(1);
	    
	    System.out.println("BrokerClient received OK from CA");

	    // read in signed cert ... 
	    

	}
	catch(Exception e) {
	    System.err.println(e.getMessage());
	    e.printStackTrace(); }      
	
	// clean up after yourself
	try {
	    out.close();
	    in.close();
	    caSocket.close(); }

	catch(Exception e) {
	    System.err.println(e.getMessage());	}  	
			
    }

    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void setUpStreams( Socket sock ) {

	try {
	    is = sock.getInputStream();
	    os = sock.getOutputStream();
	    
	    in = new BufferedReader(new InputStreamReader(is));
	    out = new PrintWriter(os,true); }
	
	catch(Exception e) {
	    e.printStackTrace();
	    System.err.println(e.getMessage()); }
    }


    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void setUpBrokerConn(  ) {

	try {	   	    	    	     
	    
	    //
	    // use your new socketFactory to create a socket to the broker	    	    	    
	   
	    brkrSocket = new Socket( brkrHost, brkrPort );	   	    

	    setUpStreams( brkrSocket );
	}

	catch(Exception e) {
	    e.printStackTrace();
	    System.err.println(e.getMessage());
	}

    }

    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void quit() {
	
	// the client should store the last nonce received from the broker
	// in a file and MAC that -- using MAC keys generated from the
	// client password (the one shared with the CA)

	System.exit(0);
    }


    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public static void main(String[] args) {
        
	Security.addProvider(new com.sun.crypto.provider.SunJCE());
	
	BrokerClient brokerClient = new BrokerClient();
	brokerClient.run();
    }   


    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void buy( String stockSym, String numShares ) {

	deal( "BUY:" + stockSym + ":" + numShares );
    }


    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void sell( String stockSym, String numShares ) {

	deal( "SELL:" + stockSym + ":" + numShares );
    }

    /**************************************************************************
     *
     *
     *
     *
     *
     **************************************************************************/
    public void deal( String order ) {

	// if is first time AND it's the first request, the BrokerClient
	// must first request a nonce from the Broker then use that
	// nonce in this transaction

	// out.println("Need a nonce...");

	// in.readLine( <the nonce value> )

	// else if it's the first time connecting for this particular
	// session but the Client has connected to the Broker previously,
	// the client should read in the stored nonce from a file,
	// verify that mac then use that nonce ...

	// subsequent trades during this session should use the last
	// nonce provided by the broker

	try {	    
	    out.println( order );	    
	}
	
	catch(Exception e) {
	    System.err.println( "BrokerClient deal() exception: " + 
				e.getMessage() );
	    e.printStackTrace(); }
    }
    
}
