//
//  BrokerClientPanel.java
//
//  GUI class for the BrokerClient Initialization.
//

package hotTip;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BrokerClientPanel extends JPanel {

    JTextField _usernameField;
    JPasswordField _passwordField;

    JTextField _clientKeyStore;
    JPasswordField _clientKeyStorePass;

    JTextField _caHostField;
    JTextField _caPortField;

    JTextField _brokerHostField;
    JTextField _brokerPortField;

    JTextField _firstTimeField;    

    JLabel _errorLabel;
    JButton _startupButton;

    BrokerClient _brokerClient;

    public BrokerClientPanel(BrokerClient brokerClient) {
	_brokerClient = brokerClient;

	try {
	    componentInit(); }
	
	catch (Exception e) {
	    System.out.println("BrokerClientPanel error: " + 
			       e.getMessage());
	    e.printStackTrace();
	}
    }
    
    void componentInit() throws Exception {
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	JLabel label;
	
	setLayout(gridBag);

	addLabel(gridBag, "BrokerClient Startup Panel", SwingConstants.CENTER, 
		 1, 0, 2, 1);
	addLabel(gridBag, "Username: ", SwingConstants.LEFT, 1, 1, 1, 1);
	addLabel(gridBag, "Password: ", SwingConstants.LEFT, 1, 2, 1, 1);
	addLabel(gridBag, "Client keystore: ", SwingConstants.LEFT, 1,3,1,1);
	addLabel(gridBag, "Client keystore pass: ", SwingConstants.LEFT,1,4,1,1);
	addLabel(gridBag, "CA hostname: ", SwingConstants.LEFT, 1, 5, 1, 1);
	addLabel(gridBag, "CA port: ", SwingConstants.LEFT, 1, 6, 1, 1);
	addLabel(gridBag, "Broker hostname: ", SwingConstants.LEFT, 1, 7, 1, 1);
	addLabel(gridBag, "Broker port: ", SwingConstants.LEFT, 1, 8, 1, 1); 
	addLabel(gridBag, "First time connecting to Broker (true/false): ", 
		 SwingConstants.LEFT, 1,9,1,1);
	
	_usernameField = new JTextField();
	addField(gridBag, _usernameField, 2, 1, 1, 1);
	
	_passwordField = new JPasswordField();
	_passwordField.setEchoChar('#');
      	addField(gridBag, _passwordField, 2, 2, 1, 1);
	
	_clientKeyStore = new JTextField();
	addField(gridBag, _clientKeyStore, 2,3,1,1);

	_clientKeyStorePass = new JPasswordField();
	_passwordField.setEchoChar('#');
	addField(gridBag, _clientKeyStorePass, 2,4,1,1);

	_caHostField = new JTextField();
      	addField(gridBag, _caHostField, 2, 5, 1, 1);
	
	_caPortField = new JTextField();
	addField(gridBag, _caPortField, 2,6,1,1);
	
	_brokerHostField = new JTextField();
	addField(gridBag, _brokerHostField, 2,7,1,1);
	
	_brokerPortField = new JTextField();
	addField(gridBag, _brokerPortField, 2,8,1,1);    
	
	_firstTimeField = new JTextField();
	addField(gridBag, _firstTimeField, 2, 9, 1, 1);

	_errorLabel = addLabel(gridBag, " ", SwingConstants.CENTER, 
			       1, 10, 2, 1);
	
	// just for testing purposs
	_errorLabel.setForeground(Color.red);
	
	_startupButton = new JButton("Startup");
	c.gridx = 1;
	c.gridy = 10;
	c.gridwidth = 2;
	gridBag.setConstraints(_startupButton, c);
	add(_startupButton);

	_startupButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    startup();
		}
	    });
    }


    JLabel addLabel(GridBagLayout gridBag, String labelStr, int align, 
		    int x, int y, int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	JLabel label = new JLabel(labelStr);
	if (align == SwingConstants.LEFT) {
	    c.anchor = GridBagConstraints.WEST;
	} else {
	    c.insets = new Insets(10, 0, 10, 0);
	}
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(label, c);
	add(label);
	
	return label;
    }
    
    void addField(GridBagLayout gridBag, JTextField field, int x, int y, 
		  int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	field.setPreferredSize(new Dimension(96, 
					     field.getMinimumSize().height));
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(field, c);
	add(field);
    }
    

    private void startup() {

	String _username = _usernameField.getText();
	char[] _password = _passwordField.getPassword();

	String _cliKSname = _clientKeyStore.getText();
	char[] _cliKSpwd =  _clientKeyStorePass.getPassword();
	
	String _caHost = _caHostField.getText();
	int _caPort = Integer.parseInt( _caPortField.getText() );

	String _brokerHost = _brokerHostField.getText();
	int _brokerPort = Integer.parseInt( _brokerPortField.getText() );
			
	String sub = _firstTimeField.getText().substring(0,1);

	boolean firstTime = sub.equalsIgnoreCase("T") ? true : false;

	_brokerClient.startup( _username, _password, 
			       _cliKSname, _cliKSpwd, 
			       _caHost, _caPort,
			       _brokerHost, _brokerPort,
			       firstTime );
	
    }
    
}
