//
//  BrokerPanel.java
//
//  GUI class for the Broker Initialization.
//

package hotTip;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BrokerPanel extends JPanel {
    
    JTextField _portField, _signedBrkrKS;

    JPasswordField _signedBrkrKSpwd;

    JLabel _errorLabel;

    JButton _startupButton;

    Broker _broker;

    public BrokerPanel(Broker broker) {

	_broker = broker;
	
	try {
	    componentInit(); }
	
	catch (Exception e) {
	    System.out.println("BrokerPanel error: " + e.getMessage());
	    e.printStackTrace(); }
    }
    
    void componentInit() throws Exception {
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	JLabel label;
	
	setLayout(gridBag);

	addLabel(gridBag, "Broker Startup Panel", SwingConstants.CENTER, 
		 1, 0, 2, 1);

	addLabel(gridBag, "Port Number: ", 
		 SwingConstants.LEFT, 1, 5, 1, 1);

	addLabel(gridBag, "Signed Broker key store name: ", 
		 SwingConstants.LEFT, 1, 6, 1, 1);
	
	addLabel(gridBag, "Signed Broker key store pwd: ",
		 SwingConstants.LEFT, 1, 7, 1, 1);
	
	_portField = new JTextField();
	addField(gridBag, _portField, 2, 5, 1, 1);
	
	_signedBrkrKS = new JTextField();
	addField(gridBag, _signedBrkrKS, 2, 6, 1, 1);
	
	_signedBrkrKSpwd = new JPasswordField();
	_signedBrkrKSpwd.setEchoChar('#');
      	addField(gridBag, _signedBrkrKSpwd, 2, 7, 1, 1);
	
	_errorLabel = addLabel(gridBag, " ", 
			       SwingConstants.CENTER, 1, 8, 2, 1);
	
	// just for testing purposs
	_errorLabel.setForeground(Color.red);
	
	_startupButton = new JButton("Startup");
	c.gridx = 1;
	c.gridy = 8;
	c.gridwidth = 2;
	gridBag.setConstraints(_startupButton, c);
	add(_startupButton);
	
	_startupButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    startup();
		}
	    });
    }

    JLabel addLabel(GridBagLayout gridBag, String labelStr, int align, 
		    int x, int y, int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	JLabel label = new JLabel(labelStr);
	if (align == SwingConstants.LEFT) {
	    c.anchor = GridBagConstraints.WEST;
	} else {
	    c.insets = new Insets(10, 0, 10, 0);
	}
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(label, c);
	add(label);

	return label;
    }

    void addField(GridBagLayout gridBag, JTextField field, int x, int y, 
		  int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	field.setPreferredSize(new Dimension(96, 
					     field.getMinimumSize().height));
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(field, c);
	add(field);
    }


    private void startup() {
      
	int _brokerPort;
	
	try {

	    _brokerPort = Integer.parseInt(_portField.getText());

	} 

	catch (NumberFormatException nfExp) {
	    
	    _errorLabel.setText("Port field is not numeric.");
	    
	    return;
	}	
	
	_broker.startup( _brokerPort,_signedBrkrKS.getText(),
			 _signedBrkrKSpwd.getPassword() );
    }
}
