//
//  BuySellPanel.java
//
//  Written by : Liz Stinson
//

package hotTip;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class BuySellPanel extends JPanel {

    JTextField stockSymField;
    JTextField numSharesField;
    
    JLabel _errorLabel;

    JButton _startupButton;

    JButton buyButton;
    JButton sellButton;
    JButton exitButton;

    BrokerClient _brokerClient;

    public BuySellPanel(BrokerClient brokerClient) {
	
	_brokerClient = brokerClient;

	try {
	    componentInit(); }
 
	catch (Exception e) {
	    System.out.println("BuySellPanel error: " + e.getMessage());
	    e.printStackTrace(); }
    }
    
    void componentInit() throws Exception {
	GridBagLayout gridBag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	JLabel label;
	
	setLayout(gridBag);
	
	addLabel(gridBag, "BrokerClient Buy/Sell Panel", SwingConstants.CENTER, 
		 1, 0, 2, 1);
	addLabel(gridBag, "Stock symbol: ", SwingConstants.LEFT, 1,1,1,1);
	addLabel(gridBag, "Num shares: ", SwingConstants.LEFT, 1,2,1,1); 
    
	stockSymField = new JTextField();
	addField(gridBag, stockSymField, 2, 1, 1, 1);
	
	numSharesField = new JTextField();
      	addField(gridBag, numSharesField, 2, 2, 1, 1);
	
	_errorLabel = addLabel(gridBag, " ", SwingConstants.CENTER, 
			       1, 3, 2, 1);

	// just for testing purposs
	_errorLabel.setForeground(Color.red);

	buyButton = new JButton("Buy");
	sellButton = new JButton("Sell");
	exitButton = new JButton("Exit");

	add(buyButton);
	add(sellButton);
	add(exitButton);

	buyButton.addActionListener( new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    buy();
		}
	    });

	sellButton.addActionListener( new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    sell();
		}
	    });

	exitButton.addActionListener( new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    quit();
		}
	    });
				 	
	c.gridx = 1;
	c.gridy = 8;
	c.gridwidth = 2;
	gridBag.setConstraints( buyButton, c );

	c.gridx = 1;
	c.gridy = 9;
	c.gridwidth = 2;
	gridBag.setConstraints( sellButton, c );

	c.gridx = 1;
	c.gridy = 10;
	c.gridwidth = 2;
	gridBag.setConstraints( exitButton, c );

    }

    JLabel addLabel(GridBagLayout gridBag, String labelStr, int align, 
		    int x, int y, int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	JLabel label = new JLabel(labelStr);
	if (align == SwingConstants.LEFT) {
	    c.anchor = GridBagConstraints.WEST;
	} else {
	    c.insets = new Insets(10, 0, 10, 0);
	}
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(label, c);
	add(label);

	return label;
    }

    void addField(GridBagLayout gridBag, JTextField field, int x, int y, 
		  int width, int height) {
	GridBagConstraints c = new GridBagConstraints();
	field.setPreferredSize(new Dimension(96, 
					     field.getMinimumSize().height));
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = width;
	c.gridheight = height;
	gridBag.setConstraints(field, c);
	add(field);
    }


    private void buy() {
			
	_brokerClient.buy( stockSymField.getText(),
			   numSharesField.getText() );
	
    }
    
    private void sell() {
	
	_brokerClient.sell( stockSymField.getText(),
			    numSharesField.getText() );
    }

    private void quit() {

	_brokerClient.quit();

    }
    
}
